function VisuMH(x1,p,gamma,Anim)
    % Inspirée de : Eugenio Bertolini (2024). Metropolis-Hastings Monte Carlo Markov chain 
% algorithm (https://www.mathworks.com/matlabcentral/fileexchange/78017-metropolis-hastings-
% monte-carlo-markov-chain-algorithm), MATLAB Central File Exchange
    [S,discarded]=MetroHastVideo(x1,p,gamma); 
    sampler = @(x) normpdf(x,0,gamma);
    x = -3.45*gamma:0.01:3.45*gamma;
    lim_a = min(-3,min(S)-1); 
    lim_b = max(3,max(S)+1); 
    fig=figure;
    set(fig, 'Position', [0, 0, 800, 300]);
    nIter = length(S);
    c=integral(p,-10,10); 
    if Anim==true
        for k = 2 : nIter-1
            s1 = subplot(1,2,1);
            histogram(S(1:k), 21, 'FaceColor', [0.0000 0.0000 1.0000],'Normalization','pdf');
            hold on 
            plot(linspace(-10,10,200),p(linspace(-10,10,200))/c,'r','Linewidth',3)
            camroll(90);
            xlim([lim_a, lim_b]);
            ylim([0,1]); 
            title(['distribution des x_j, j=1,...,' num2str(k)])
            s2 = subplot(1,2,2);
            hold on;
            plot(S(1:k), 'Color', [0.9000 0.6000 0.2000], 'LineWidth', 1.2);
            plot(k/6 * sampler(x) / sampler(0) + k+1, x + S(k), 'b');
            if discarded(k+1) == 0
                plot([k, k+1, k/6 * sampler(S(k+1) - S(k)) / sampler(0) + k+1], [S(k:k+1); S(k+1)], 'Color', 'g', 'LineWidth', 1.5);
                plot(k+1, S(k+1), 'go', 'LineWidth', 3);
            else
                plot([k, k+1, k/6 * sampler(discarded(k+1) - S(k)) / sampler(0) + k+1], [S(k), discarded(k+1), discarded(k+1)], 'Color', 'r', 'LineWidth', 1.5);
                plot(k+1, discarded(k+1), 'rx', 'LineWidth', 3);
            end
            xlim([1, k*1.2 + 5]);
            ylim([lim_a, lim_b]);
            xlabel('j')
            ylabel('x_j')
            title(['x_j, j=' num2str(k)])
            s1.Position = s1.Position + [-0.08 0 -0.16 0];
            s2.Position = s2.Position + [-0.28 0 0.33 0];
            if k<10
                pause(1)
            else
                pause(0.05)
            end
        end
    end
    s1=subplot(1,2,1);
    histogram(S, 21, 'FaceColor', [0.0000 0.0000 1.0000],'Normalization','pdf');
    hold on 
    plot(linspace(-10,10,200),p(linspace(-10,10,200))/c,'r','Linewidth',3)
    camroll(90);
    xlim([lim_a, lim_b]);
    ylim([0,1]); 
    title(['distribution des x_j, j=1,...,' num2str(length(S))])
    s2=subplot(1,2,2);
    hold on;
    plot(S, 'go','LineWidth', 3);
    non_zero_elements = discarded(discarded ~= 0);
    indices_non_zero = find(discarded ~= 0);
    plot(indices_non_zero,non_zero_elements,'rx','LineWidth', 3);
    xlim([1, length(S)*1.2 + 5]);
    ylim([lim_a, lim_b]);
    xlabel('j')
    ylabel('x_j')
    title(['x_j, j=' num2str(length(S))])
    s1.Position = s1.Position + [-0.08 0 -0.16 0];
    s2.Position = s2.Position + [-0.28 0 0.33 0];
end


function [x, propx] = MetroHastVideo(x1, p, gamma)
    nIter=300; 
    x = zeros(nIter,1);
    propx = zeros(nIter,1);
    x(1) = x1;
    for k = 2:nIter
        current = x(k - 1);
        proposed = normrnd(current, gamma);
        A = p(proposed) / p(current);
        if(rand < A)
             x(k) = proposed;
        else
             x(k) = current;
             propx(k) = proposed;
        end
    end
end